#lang racket/base

(require (only-in "../aux.rkt"
           list-ref/default))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sum of Euler's totient function

;; Own implementation of gcd (Pycket's builtin is too slow, and might not
;; trace correctly)
(define (gcd a b)
  (if (= b 0)
      a
      (gcd b (modulo a b))))

(define (totient n)
  (for/sum ([k (in-range n)]) (if (= (gcd n k) 1) 1 0)))


;; Sum of `totient` over a sequence integers `ns`.
(define (sum-totient ns)
  (apply + (for/list ([n ns]) (totient n))))


;; Det init trace cache by calling sum-totient on fixed random list of ints
(define random-data
  '(987 633 583 261 348 128 860 333 953 482 37 329 270 601 767 977 351
    545 296 672 924 475 274 844 61 327 516 753 50 154 311 652 499 665
    40 932 22 935 828 191 69 220 196 657 167 375 783 362 662 9))
(unless (eq? (sum-totient random-data) 14574)
  (displayln "PANIC!"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; "main" script
;;
;; Arguments
;; 1) lower [number? = 6001]
;; 2) len   [number? = 4000]

(define args  (vector->list (current-command-line-arguments)))
(unless (null? args)
  (define lower (string->number (list-ref/default "6001" args 0)))
  (define len   (string->number (list-ref/default "4000" args 1)))
  (define ns    (for/list ([n (in-range lower (+ lower len))]) n))
  (printf "{seq} (sum-totient [~a..~a])\n" lower (sub1 (+ lower len)))
  (define result (time (sum-totient ns)))
  (printf "~a\n" result))
